function perceptronToPlane(algorithmState, UIConfig, ThreeJSObjects) {
	if (algorithmState.W[2] != 0) {
		var v = ThreeJSObjects.hyperplane.position.clone();
		v.add(new THREE.Vector3(algorithmState.W[0], algorithmState.W[1], algorithmState.W[2]));
		ThreeJSObjects.hyperplane.lookAt(v);
		var intercept = (- algorithmState.b / algorithmState.W[2]) * UIConfig.unitLength;
		ThreeJSObjects.hyperplane.position.x = 0;
		ThreeJSObjects.hyperplane.position.y = 0;
		ThreeJSObjects.hyperplane.position.z = intercept;
		
		//update normal vector
		//var position = new THREE.Vector3(0,0,0);
		var position = ThreeJSObjects.hyperplane.position;
		//ThreeJSObjects.normalVector.position.set(position);
		ThreeJSObjects.normalVector.position.set(position.x, position.y, position.z);
		var direction = new THREE.Vector3(algorithmState.W[0], algorithmState.W[1], algorithmState.W[2]);
		ThreeJSObjects.normalVector.setDirection(direction.normalize());

		//set light position
		v = direction.normalize();
		v = v.multiplyScalar(UIConfig.lightPlaneDistance);
		ThreeJSObjects.redLight.position.set(v.x, v.y, v.z);
		ThreeJSObjects.blueLight.position.set(-v.x, -v.y, -v.z);
	}
}

function setControls(camera){	
	controls = new THREE.TrackballControls(camera, document.getElementById('container').firstChild);
	controls.rotateSpeed = 1.0;
	controls.zoomSpeed = 1.2;
	controls.panSpeed = 0.8;
	controls.noZoom = false;
	controls.noPan = false;
	controls.staticMoving = true;
	controls.dynamicDampingFactor = 0.3;
	controls.keys = [ 65, 83, 68 ];	
	controls.addEventListener('change', render);	
	return controls;
}

function RenderDataset(dataset, algorithmState, UIConfig){
	var dataObjects = [];	
	for (var i=0; i < dataset.examples.length; i++) {
		var exampleGeometry = new THREE.SphereGeometry(7, 32, 16);
		var material = dataset.classes[i] ? UIConfig.materials.classOne : UIConfig.materials.classZero;
		var exampleMesh = new THREE.Mesh(exampleGeometry, material);						
		exampleMesh.position.set(dataset.examples[i][0] * UIConfig.unitLength,
					(dataset.examples[i][1])* UIConfig.unitLength,
					dataset.examples[i][2] * UIConfig.unitLength);
		UIConfig.scene.add(exampleMesh);		
		dataObjects.push(exampleMesh);
	}
	return dataObjects;	
}

//TO-DO

function clearHighlights(dataset, UIConfig, ThreeJSObjects){
	for (var i=0; i < dataset.examples.length; i++) {
		if (dataset.classes[i] == 0)
			ThreeJSObjects.dataObjects[i].material = UIConfig.materials.classZero;
		else
			ThreeJSObjects.dataObjects[i].material = UIConfig.materials.classOne;	
	}
}

function highlightExample(dataset, algorithmState, UIConfig, ThreeJSObjects){	
	var i = algorithmState.current_example_index;
	if (dataset.classes[i] == 0)
		ThreeJSObjects.dataObjects[i].material = UIConfig.materials.classZeroHighlight;
	else
		ThreeJSObjects.dataObjects[i].material = UIConfig.materials.classOneHighlight;	
}

function highlightOnlyExample(dataset, algorithmState, UIConfig, ThreeJSObjects){	
	clearHighlights(dataset, UIConfig, ThreeJSObjects);
	highlightExample(dataset, algorithmState, UIConfig, ThreeJSObjects);
}

function createThreeJSObjects(dataset, algorithmState, UIConfig){
	var useTextureForPlane = false;
	var back_geo = new THREE.SphereGeometry(9000, 32, 16);	
	var back_material = new THREE.MeshBasicMaterial({ color:0x9999ff, side: THREE.DoubleSide });
	
	var back_mesh = new THREE.Mesh(back_geo, back_material);
	UIConfig.scene.add(back_mesh);				
	
	var plane_geo = new THREE.PlaneGeometry(UIConfig.planeSize, UIConfig.planeSize, 10, 10);
	var floorTexture = new THREE.ImageUtils.loadTexture('images/checkerboard.jpg');
	floorTexture.wrapS = floorTexture.wrapT = THREE.RepeatWrapping; 
	floorTexture.repeat.set(20, 20);
	//var plane_color = 0x339933; 
	var plane_color = 0x444444; 
	if (useTextureForPlane)
		var plane_material = new THREE.MeshBasicMaterial({ map: floorTexture, side: THREE.DoubleSide, transparent: true, opacity: 0.8});
	else	
		//var plane_material = new THREE.MeshBasicMaterial({ color: 0x555555, side: THREE.DoubleSide, transparent: true, opacity: 0.3});					
		var plane_material = new THREE.MeshPhongMaterial({ color: plane_color,  specular: 0x444444, shininess: 0, ambient: 0x000000,
						side: THREE.DoubleSide, transparent: true, opacity: 1.0, metal:false});	
	plane_mesh = new THREE.Mesh(plane_geo, plane_material);
	UIConfig.scene.add(plane_mesh);
	
	var dir = new THREE.Vector3(0.5, 0.5, 0.5); //dummy
	var origin = new THREE.Vector3(0, 0, 0); //dummy
	var length = UIConfig.hyperplaneLength;
	var hex = 0xffff00;
	var arrowHelper = new THREE.ArrowHelper( dir, origin, length, hex ); 
	UIConfig.scene.add(arrowHelper);
	var dataObjects = RenderDataset(dataset, algorithmState, UIConfig);
	lights = setUpLights(UIConfig.scene);				
	return {hyperplane: plane_mesh, normalVector: arrowHelper, dataObjects: dataObjects,
			redLight: lights.redLight, blueLight: lights.blueLight};
}

function setUpLights(scene){
	// lights				
	light = new THREE.DirectionalLight(0xffffff);
	light.position.set(1, 1, 1);
	//scene.add(light);				
	light = new THREE.DirectionalLight(0xffffff);
	light.position.set(-1, -1, -1);
	//scene.add(light);	
	//ambientLight = new THREE.AmbientLight(0x888888);
	var bright = 2.5;
	var reach = 0;
	var light_distance=70;
	redLight = new THREE.PointLight(0xff3333,bright,reach);
	//redLight.position.set(light_distance, light_distance, light_distance);
	scene.add(redLight);
	blueLight = new THREE.PointLight(0x3333ff,bright,reach);
	//blueLight.position.set(-light_distance,-light_distance,-light_distance);
	scene.add(blueLight);	
	light = new THREE.AmbientLight(0x404040);
	//scene.add(light);
	return {redLight: redLight, blueLight: blueLight};
}

function createUIConfig(scene, controls, camera) {
	var classOneColor = 0xff2222; //red
	var classZeroColor = 0x2222ff; //blue
	var classOneColorHighlight = 0xffff22; //red
	var classZeroColorHighlight = 0x22ffff; //blue

	var materials = {
	 classOne : new THREE.MeshPhongMaterial({color: classOneColor, specular: classOneColor}), 
	 classZero : new THREE.MeshPhongMaterial({color: classZeroColor, specular: classZeroColor}), 				
	 classOneHighlight : new THREE.MeshPhongMaterial({color: classOneColorHighlight, specular: classOneColorHighlight}),
	 classZeroHighlight : new THREE.MeshPhongMaterial({color: classZeroColorHighlight, specular: classZeroColorHighlight})
	}; 				
	
	return {scene: scene, camera:camera, controls:controls, unitLength:10, 
			classOneColor: classOneColor, classZeroColor: classZeroColor, planeSize: 400,
			hyperplaneLength: 200, materials : materials, lightPlaneDistance : 200};
}

