function addEventListeners(){
	$("#reset").click(reset);
	$("#reset").click(refreshHyperplane);
	$("#next").click(next);
	$("#nextIteration").click(nextIteration);	
}

function refreshHyperplane(){
	perceptronToPlane(app.algorithmState, app.UIConfig, app.ThreeJSObjects);
	render();
}

function reset(){
	//dataset initialization	
	if (app.pseudo === true){	
		//app.algorithmConfig = createAlgorithmConfig(0.1, 10, 3);
		app.algorithmConfig = createAlgorithmConfig(0.1, 50, 3);
		//app.algorithmState = createAlgorithmState(app.algorithmConfig);
		W = [0.8, 0.2, 0.7];
		b = -0.5;
		app.algorithmState = createAlgorithmStateWithValues(app.algorithmConfig, W, b);
		var datasetConfig = createDatasetConfig([3, 3], 10); //dummy
		//app.dataset = createDataset(datasetConfig);
		examples = [[-0.32521068483650595, 0.9117357067972245, 2.682042035355395], 
					[6.147267552763111, 8.178539400879133, 2.079918435402182], 
					[2.2838567553755214, 0.4945748831282426, 2.8772291996781965], 
					[6.611305336847088, 2.396143898974934, 8.048457188225832], 				
					[3.9450556197005, 4.068331320958512, -1.7903946805871902], 
					[6.125976130221841, 4.41789509868661, 8.198545258968707]];
		classes = [1,0,1,0,1,0];
		app.dataset = createDatasetWithValues(examples, classes, datasetConfig);
	}
	else{
		var alpha = $("#alpha").val();
		var ITE_MAX = $("#ITE_MAX").val();
		var examplesClassZero = $("#examplesClassZero").val();
		var examplesClassOne = $("#examplesClassOne").val();
		var centerDistance = $("#classSeparation").val();
		app.algorithmConfig = createAlgorithmConfig(alpha, ITE_MAX, 3);				
		app.algorithmState = createAlgorithmState(app.algorithmConfig);
		var datasetConfig = createDatasetConfig([examplesClassZero, examplesClassOne], centerDistance); //dummy;
		app.dataset = createDataset(datasetConfig);	
	}
	if 	(typeof app.ThreeJSObjects !== "undefined")
		clearSceneObjects(app.UIConfig, app.ThreeJSObjects);
	app.ThreeJSObjects = createThreeJSObjects(app.dataset, app.algorithmState, app.UIConfig);	
	app.animationState = AnimationState.Initializing;
	initializeAnimationState();	
	enableExecutionControls();
}

function clearSceneObjects(UIConfig, ThreeJSObjects){
	UIConfig.scene.remove(ThreeJSObjects.hyperplane);
	UIConfig.scene.remove(ThreeJSObjects.normalVector);
	UIConfig.scene.remove(ThreeJSObjects.redLight);
	UIConfig.scene.remove(ThreeJSObjects.blueLight);
	for (var i=0; i < ThreeJSObjects.dataObjects.length; i++)	
		UIConfig.scene.remove(ThreeJSObjects.dataObjects[i]);
}

function nextIteration(){	
	if (algorithmHasFinished(app.algorithmState, app.algorithmConfig) && iterationHasFinished(app.algorithmState, app.dataset.examples.length)){
		app.animationState = AnimationState.Finished;
		initializeAnimationState();
		refreshHyperplane();		
		return;
	}	
	if (iterationHasFinished(app.algorithmState, app.dataset.examples.length)){
		nextExample(app.algorithmState, app.dataset.examples.length);
	}
	while (! iterationHasFinished(app.algorithmState, app.dataset.examples.length)){
		update(app.algorithmState, app.dataset, app.algorithmConfig);					
		nextExample(app.algorithmState, app.dataset.examples.length);
	}	
	app.animationState = AnimationState.CheckStoppingConditions;	
	initializeAnimationState();	
	refreshHyperplane();	
}

//state machine implementation and algorithm execution
function afterNext(){
	switch (app.animationState) {
		case AnimationState.Initializing:
			//refresh UI
			app.animationState = AnimationState.CheckStoppingConditions;
			break;
		case AnimationState.BeginningIteration:
			app.animationState = AnimationState.ComparingOutput;
			
			break;		
		case AnimationState.CheckStoppingConditions:
			//refresh UI
			if (algorithmHasFinished(app.algorithmState, app.algorithmConfig)){
				app.animationState = AnimationState.Finished;
			}
			else {				
				if (iterationHasFinished(app.algorithmState, app.dataset.examples.length)) {
					beginIteration(app.algorithmState);
				}
				app.animationState = AnimationState.BeginningIteration;
			}
			break;
		case AnimationState.ComparingOutput:
			//refresh UI
			if (isCurrentExampleCorrectlyClassified(app.algorithmState, app.dataset)){			
				nextExample(app.algorithmState, app.dataset.examples.length);
				if (iterationHasFinished(app.algorithmState, app.dataset.examples.length)) {
					app.animationState = AnimationState.CheckStoppingConditions;					
				}
				else {
					app.animationState = AnimationState.BeginningIteration;					
				}
			}
			else {
				app.animationState = AnimationState.UpdatingWeights;
			}			
			break;
		case AnimationState.UpdatingWeights:
			update(app.algorithmState, app.dataset, app.algorithmConfig);			
			//refresh UI
			nextExample(app.algorithmState, app.dataset.examples.length);
			if (iterationHasFinished(app.algorithmState, app.dataset.examples.length))
				app.animationState = AnimationState.CheckStoppingConditions;
			else
				app.animationState = AnimationState.BeginningIteration;
			break;
		case AnimationState.Finished:
			//do nothing			
			break;		
	}
}

function disableExecutionControls(){
	$("#next").prop('disabled', true);
	$("#nextIteration").prop('disabled', true);
}
function enableExecutionControls(){
	$("#next").prop('disabled', false);
	$("#nextIteration").prop('disabled', false);
}

function initializeAnimationState(){
	//upon entering state
	switch (app.animationState) {
		case AnimationState.Initializing:
			perceptronToPlane(app.algorithmState, app.UIConfig, app.ThreeJSObjects);
			break;
		case AnimationState.ComparingOutput:
			//refresh UI
			highlightOnlyExample(app.dataset, app.algorithmState, app.UIConfig, app.ThreeJSObjects);
			break;
		case AnimationState.UpdatingWeights:
			highlightOnlyExample(app.dataset, app.algorithmState, app.UIConfig, app.ThreeJSObjects);
			break;
		case AnimationState.Finished:
			disableExecutionControls();
			break;		
		default:
			clearHighlights(app.dataset, app.UIConfig, app.ThreeJSObjects); 
	}
	highlightCode();
	updateVariableLabels();	
}

function next(){
	//after pressing "next"
	afterNext();
	initializeAnimationState();
	refreshHyperplane();
}

function highlightCode(){
	var allPre = $("#code pre");
	allPre.removeClass("highlight");
	$("#" + app.animationState + "Block").addClass("highlight");
	
	var allPre = $("#pseudocode pre");
	allPre.removeClass("highlight");
	$("#" + app.animationState + "Block").addClass("highlight");
}

function updateVariableLabels(){
	$("#iterations").text(app.algorithmState.iteration);
	$("#errors").text(app.algorithmState.error_count);
	$("#example_index").text(app.algorithmState.current_example_index + 1);
	$("#ITE_MAX").val(app.algorithmConfig.ITE_MAX);
	$("#alpha").val(app.algorithmConfig.alpha);
	$("#examplesClassOne").val(app.dataset.datasetConfig.examples_per_class[1]);
	$("#examplesClassOne").css("color", "#" + app.UIConfig.classOneColor.toString(16));	
	$("#examplesClassZero").val(app.dataset.datasetConfig.examples_per_class[0]);	
	$("#examplesClassZero").css("color", "#" + app.UIConfig.classZeroColor.toString(16));	
	$("#classSeparation").val(app.dataset.datasetConfig.center_distance);
	
	$("#alpha").val(app.algorithmConfig.alpha);
	var str = "[" + app.algorithmState.b.toFixed(4);
	for (var i=1; i < app.algorithmState.W.length; i++) 
		str += ", " + app.algorithmState.W[i - 1].toFixed(4);
	str += "]";
	$("#weights").text(str);
}

function createAlgorithmConfig(alpha, ITE_MAX, inputDims){
	return {alpha: alpha, ITE_MAX:ITE_MAX, inputDims: inputDims};
}

function createAlgorithmState(algorithmConfig){
	var algorithm_state = {iteration :0,
						   current_example_index:0,
						   error_count:1};
	algorithm_state.W = [];
	for (var i=0; i < algorithmConfig.inputDims; i++)
		algorithm_state.W[i] = Math.random() - 0.5;	
	algorithm_state.b = Math.random() - 0.5;
	return algorithm_state;
}

function createAlgorithmStateWithValues(algorithmConfig, W, b){
	return {iteration :0,
		   current_example_index:0,
		   error_count:1,
		   W: W,
		   b: b};
}

function createDatasetConfig(examples_per_class, center_distance){
	return {examples_per_class: examples_per_class, 
		center_distance: center_distance};
}

function createDataset(datasetConfig){
	var N = datasetConfig.examples_per_class[0] + datasetConfig.examples_per_class[1];	

	var ClassOneDispersion = 8;
	var ClassZeroDispersion = 8;
	var r = Math.random();
	var d = datasetConfig.center_distance / 2;
	var classOneCenter = {x: d + r, y: d + r, z: d + r};
	var classZeroCenter = {x: -d + r, y: -d + r, z: -d + r};
	console.log(d + " " + r + " " + classOneCenter);
	//TO-DO: Use center distance
	examples = [];
	classes = [];				
	for (var i=0; i < datasetConfig.examples_per_class[1]; i++) {
		example = [];
		example[0] = classOneCenter.x + Math.random() * ClassOneDispersion;
		example[1] = classOneCenter.y + Math.random() * ClassOneDispersion;
		example[2] = classOneCenter.z + Math.random() * ClassOneDispersion;
		classes.push(1);
		examples.push(example);
	}
	for (var i=0; i < datasetConfig.examples_per_class[0]; i++) {	
		example = [];	
		example[0] = classZeroCenter.x + Math.random() * ClassZeroDispersion;
		example[1] = classZeroCenter.y + Math.random() * ClassZeroDispersion;
		example[2] = classZeroCenter.z + Math.random() * ClassZeroDispersion;									
		examples.push(example);
		classes.push(0);
	}
	return {examples: examples, classes: classes, datasetConfig: datasetConfig};
}

function createDatasetWithValues(examples, classes, datasetConfig){
	return {examples: examples, classes: classes, datasetConfig: datasetConfig};
}

//before doing action
AnimationState = {Initializing: 'Initializing', 
	CheckStoppingConditions: 'CheckStoppingConditions', 
	BeginningIteration: 'BeginningIteration', //the beginning of the "for" for the examples
	ComparingOutput: 'ComparingOutput',
	UpdatingWeights: 'UpdatingWeights',
	Finished: 'Finished'};
	
	
	